@@ -1,7 +1,7 @@
 '''
 Introduces setup and teardown routines suitable for testing Vagrant.
 
-Note that the tests can take few minutes to run because of the time 
+Note that the tests can take few minutes to run because of the time
 required to bring up/down the VM.
 
 Most test functions (decorated with `@with_setup`) will actually bring the VM
@@ -11,12 +11,12 @@ suite. The other approach (bringing the VM up once and packing all tests
 together) is less extensible.  With few tests it may work, but when the test
 suite grows it becomes a problem.
 
-In order to mitigate the inconvenience, each test method actually 
+In order to mitigate the inconvenience, each test method actually
 encapsulates few related tests. Aside from making the execution time shorter
 it also adds to readability.
 
-Before the first test a base box is added to Vagrant under the name 
-TEST_BOX_NAME. This box is not deleted after the test suite runs in order 
+Before the first test a base box is added to Vagrant under the name
+TEST_BOX_NAME. This box is not deleted after the test suite runs in order
 to avoid downloading of the box file on every run.
 '''
 
@@ -35,9 +35,9 @@ import vagrant
 # location of a test file on the created box by provisioning in vm_Vagrantfile
 TEST_FILE_PATH = '/home/vagrant/python_vagrant_test_file'
 # location of Vagrantfiles used for testing.
-MULTIVM_VAGRANTFILE = os.path.join(os.path.dirname(os.path.abspath(__file__)), 
+MULTIVM_VAGRANTFILE = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                    'vagrantfiles', 'multivm_Vagrantfile')
-VM_VAGRANTFILE = os.path.join(os.path.dirname(os.path.abspath(__file__)), 
+VM_VAGRANTFILE = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                    'vagrantfiles', 'vm_Vagrantfile')
 # the names of the vms from the multi-vm Vagrantfile.
 VM_1 = 'web'
@@ -84,7 +84,7 @@ def list_box_names():
 
 def setup():
     '''
-    Creates the directory used for testing and sets up the base box if not 
+    Creates the directory used for testing and sets up the base box if not
     already set up.
 
     Creates a directory in a temporary location and checks if there is a base
@@ -127,7 +127,7 @@ def teardown_vm():
     '''
     try:
         subprocess.check_call(
-            'vagrant destroy -f'.format(TEST_BOX_NAME), 
+            'vagrant destroy -f'.format(TEST_BOX_NAME),
             cwd=TD, shell=True)
     except subprocess.CalledProcessError:
         pass
@@ -152,6 +152,21 @@ def teardown_multivm():
         os.unlink(os.path.join(TD, "Vagrantfile"))
 
 
+@with_setup(setup_vm, teardown_vm)
+def test_plugin_list_parsing():
+    '''
+    Test the parsing the output of the `vagrant plugin list` command.
+    '''
+    # listing should match output generated by `vagrant plugin list`.
+    listing = '''sahara (0.0.16)
+vagrant-login (1.0.1, system)
+'''
+    goal = [{'name': 'sahara', 'version': '0.0.16', 'system': False},
+            {'name': 'vagrant-login', 'version': '1.0.1', 'system': True}]
+    v = vagrant.Vagrant(TD)
+    parsed = [v._parse_plugin_list_line(l) for l in listing.splitlines()]
+    assert goal == parsed, 'The parsing of the test listing did not match the goal.\nlisting={!r}\ngoal={!r}\nparsed_listing={!r}'.format(listing, goal, parsed)
+
 
 @with_setup(setup_vm, teardown_vm)
 def test_vm_status():
@@ -163,7 +178,7 @@ def test_vm_status():
     command = 'vagrant up'
     subprocess.check_call(command, cwd=TD, shell=True)
     assert v.RUNNING in v.status().values(), "After going up status should be vagrant.RUNNING"
-    
+
     command = 'vagrant halt'
     subprocess.check_call(command, cwd=TD, shell=True)
     assert v.POWEROFF in v.status().values(), "After halting status should be vagrant.POWEROFF"
@@ -171,7 +186,7 @@ def test_vm_status():
     command = 'vagrant destroy -f'
     subprocess.check_call(command, cwd=TD, shell=True)
     assert v.NOT_CREATED in v.status().values(), "After destroying status should be vagrant.NOT_CREATED"
-    
+
 
 @with_setup(setup_vm, teardown_vm)
 def test_vm_lifecycle():
@@ -185,16 +200,16 @@ def test_vm_lifecycle():
     os.unlink(os.path.join(TD, 'Vagrantfile'))
     v.init(TEST_BOX_NAME)
     assert v.NOT_CREATED in v.status().values()
-        
+
     v.up()
     assert v.RUNNING in v.status().values()
-    
+
     v.suspend()
     assert v.SAVED in v.status().values()
-    
+
     v.halt()
     assert v.POWEROFF in v.status().values()
-    
+
     v.destroy()
     assert v.NOT_CREATED in v.status().values()
 
@@ -211,26 +226,26 @@ def test_vm_config():
     parsed_config = dict(line.strip().split(None, 1) for line in
                             ssh_config.splitlines() if line.strip() and not
                             line.strip().startswith('#'))
-    
+
     user = v.user()
-    expected_user = parsed_config["User"] 
+    expected_user = parsed_config["User"]
     eq_(user, expected_user)
-    
+
     hostname = v.hostname()
     expected_hostname = parsed_config["HostName"]
     eq_(hostname, expected_hostname)
-    
+
     port = v.port()
     expected_port = parsed_config["Port"]
     eq_(port, expected_port)
-    
+
     user_hostname = v.user_hostname()
     eq_(user_hostname, "{}@{}".format(expected_user, expected_hostname))
-    
+
     user_hostname_port = v.user_hostname_port()
-    eq_(user_hostname_port,  
+    eq_(user_hostname_port,
         "{}@{}:{}".format(expected_user, expected_hostname, expected_port))
-    
+
     keyfile = v.keyfile()
     eq_(keyfile, parsed_config["IdentityFile"])
 
@@ -238,9 +253,9 @@ def test_vm_config():
 @with_setup(setup_vm, teardown_vm)
 def test_vm_sandbox_mode():
     '''
-    Test methods for enabling/disabling the sandbox mode 
+    Test methods for enabling/disabling the sandbox mode
     and committing/rolling back changes.
-    
+
     This depends on the Sahara gem (gem install sahara).
     '''
     command = "vagrant sandbox status"
@@ -251,48 +266,48 @@ def test_vm_sandbox_mode():
     # output = subprocess.check_output(command, cwd=TD, shell=True)
     sahara_installed = True if not "Usage" in output else False
     eq_(sahara_installed, True, "Sahara gem should be installed")
-    
+
     if sahara_installed:
         v = vagrant.SandboxVagrant(TD)
-        
+
         sandbox_status = v.sandbox_status()
         assert "unknown" in v.status().values(), "Before the VM goes up the status should be 'unknown', " + "got:'{}'".format(sandbox_status)
-        
+
         v.up()
         sandbox_status = v.sandbox_status()
         assert "off" in v.status().values(), "After the VM goes up the status should be 'off', " + "got:'{}'".format(sandbox_status)
-        
+
         v.sandbox_on()
         sandbox_status = v.sandbox_status()
         assert "on" in v.status().values(), "After enabling the sandbox mode the status should be 'on', " + "got:'{}'".format(sandbox_status)
-        
+
         v.sandbox_off()
         sandbox_status = v.sandbox_status()
         assert "off" in v.status().values(), "After disabling the sandbox mode the status should be 'off', " + "got:'{}'".format(sandbox_status)
-        
+
         v.sandbox_on()
         v.halt()
         sandbox_status = v.sandbox_status()
         assert "on" in v.status().values(), "After halting the VM the status should be 'on', " + "got:'{}'".format(sandbox_status)
-        
+
         v.up()
         sandbox_status = v.sandbox_status()
         assert "on" in v.status().values(), "After bringing the VM up again the status should be 'on', " + "got:'{}'".format(sandbox_status)
-        
+
         test_file_contents = _read_test_file(v)
         print test_file_contents
         eq_(test_file_contents, None, "There should be no test file")
-        
+
         _write_test_file(v, "foo")
         test_file_contents = _read_test_file(v)
         print test_file_contents
         eq_(test_file_contents, "foo", "The test file should read 'foo'")
-        
-        v.sandbox_rollback()            
+
+        v.sandbox_rollback()
         test_file_contents = _read_test_file(v)
         print test_file_contents
         eq_(test_file_contents, None, "There should be no test file")
-        
+
         _write_test_file(v, "foo")
         test_file_contents = _read_test_file(v)
         print test_file_contents
@@ -302,22 +317,22 @@ def test_vm_sandbox_mode():
         test_file_contents = _read_test_file(v)
         print test_file_contents
         eq_(test_file_contents, "bar", "The test file should read 'bar'")
-        
-        v.sandbox_rollback()            
+
+        v.sandbox_rollback()
         test_file_contents = _read_test_file(v)
         print test_file_contents
         eq_(test_file_contents, "foo", "The test file should read 'foo'")
-        
+
         sandbox_status = v._parse_vagrant_sandbox_status("Usage: ...")
         eq_(sandbox_status, "not installed", "When 'vagrant sandbox status'" +
             " outputs vagrant help status should be 'not installed', " +
             "got:'{}'".format(sandbox_status))
-        
+
         v.destroy()
         sandbox_status = v.sandbox_status()
         assert "unknown" in v.status().values(), "After destroying the VM the status should be 'unknown', " + "got:'{}'".format(sandbox_status)
 
-        
+
 @with_setup(setup_vm, teardown_vm)
 def test_boxes():
     '''
@@ -372,11 +387,11 @@ def test_provisioning():
     the vm with the contents 'foo'.
     '''
     v = vagrant.Vagrant(TD)
-    
+
     v.up(no_provision=True)
     test_file_contents = _read_test_file(v)
     eq_(test_file_contents, None, "There should be no test file after up()")
-    
+
     v.provision()
     test_file_contents = _read_test_file(v)
     print "Contents: {}".format(test_file_contents)
@@ -432,7 +447,7 @@ def test_multivm_config():
                             line.strip().startswith('#'))
 
     user = v.user(vm_name=VM_1)
-    expected_user = parsed_config["User"] 
+    expected_user = parsed_config["User"]
     eq_(user, expected_user)
 
     hostname = v.hostname(vm_name=VM_1)
@@ -483,5 +498,3 @@ def _read_test_file(v):
         return output.strip()
     except subprocess.CalledProcessError:
         return None
-
-
