@@ -38,7 +38,7 @@ import subprocess
 
 # python package version
 # should match r"^__version__ = '(?P<version>[^']+)'$" for setup.py
-__version__ = '0.4.3'
+__version__ = '0.4.4'
 
 
 def which(program):
@@ -118,7 +118,12 @@ class Vagrant(object):
     POWEROFF = 'poweroff'  # vagrant halt
     ABORTED = 'aborted'  # The VM is in an aborted state
     SAVED = 'saved' # vagrant suspend
-    STATUSES = (RUNNING, NOT_CREATED, POWEROFF, ABORTED, SAVED)
+
+    # LXC statuses
+    STOPPED = 'stopped'
+    FROZEN = 'frozen'
+
+    STATUSES = (RUNNING, NOT_CREATED, POWEROFF, ABORTED, SAVED, STOPPED, FROZEN)
 
     BASE_BOXES = {
         'ubuntu-Lucid32': 'http://files.vagrantup.com/lucid32.box',
@@ -137,6 +142,16 @@ class Vagrant(object):
         self.root = os.path.abspath(root) if root is not None else os.getcwd()
         self._cached_conf = {}
 
+    def version(self):
+        '''
+        Return the installed vagrant version, as a string, e.g. '1.5.0'
+        '''
+        output = self._run_vagrant_command('--version')
+        m = re.search(r'^Vagrant (?P<version>.+)$', output)
+        if m is None:
+            raise Exception('Failed to parse vagrant --version output. output={!r}'.format(output))
+        return m.group('version')
+
     def init(self, box_name=None, box_url=None, **kwargs):
         '''
         From the Vagrant docs:
@@ -153,16 +168,35 @@ class Vagrant(object):
         '''
         self._run_vagrant_command('init', box_name, box_url, **kwargs)
 
-    def up(self, no_provision=False, provider=None, vm_name=None, **kwargs):
+    def up(self, no_provision=False, provider=None, vm_name=None,
+           provision=None, provision_with=None, **kwargs):
         '''
         Launch the Vagrant box.
+        provision_with: optional list of provisioners to enable.
+        provider: Back the machine with a specific provider
+        no_provision: if True, disable provisioning.  Same as 'provision=False'.
+        provision: optional boolean.  Enable or disable provisioning.  Default
+          behavior is to use the underlying vagrant default.
+        Note: If provision and no_provision are not None, no_provision will be
+        ignored.
         '''
-        no_provision_arg = '--no-provision' if no_provision else None
         provider_arg = '--provider=%s' % provider if provider else None
+        prov_with_arg = None if provision_with is None else '--provision-with'
+        providers_arg = None if provision_with is None else ','.join(provision_with)
+
+        # For the sake of backward compatibility, no_provision is allowed.
+        # However it is ignored if provision is set.
+        if provision is not None:
+            no_provision = None
+        no_provision_arg = '--no-provision' if no_provision else None
+        provision_arg = None if provision is None else '--provision' if provision else '--no-provision'
+
         self._run_vagrant_command('up',
                                   vm_name,
                                   no_provision_arg,
+                                  provision_arg,
                                   provider_arg,
+                                  prov_with_arg, providers_arg,
                                   **kwargs)
         try:
             self.conf(vm_name=vm_name)  # cache configuration
@@ -171,6 +205,40 @@ class Vagrant(object):
             # however vm_name is required for conf() or ssh_config().
             pass
 
+    def provision(self, vm_name=None, provision_with=None, **kwargs):
+        '''
+        Runs the provisioners defined in the Vagrantfile.
+        vm_name: optional VM name string.
+        provision_with: optional list of provisioners to enable.
+          e.g. ['shell', 'chef_solo']
+        '''
+        prov_with_arg = None if provision_with is None else '--provision-with'
+        providers_arg = None if provision_with is None else ','.join(provision_with)
+        self._run_vagrant_command('provision', vm_name, prov_with_arg,
+                                  providers_arg, **kwargs)
+
+    def reload(self, vm_name=None, provision=None,
+               provision_with=None, **kwargs):
+        '''
+        Quoting from Vagrant docs:
+        > The equivalent of running a halt followed by an up.
+
+        > This command is usually required for changes made in the Vagrantfile to take effect. After making any modifications to the Vagrantfile, a reload should be called.
+
+        > The configured provisioners will not run again, by default. You can force the provisioners to re-run by specifying the --provision flag.
+
+        provision: optional boolean.  Enable or disable provisioning.  Default
+          behavior is to use the underlying vagrant default.
+        provision_with: optional list of provisioners to enable.
+          e.g. ['shell', 'chef_solo']
+        '''
+        prov_with_arg = None if provision_with is None else '--provision-with'
+        providers_arg = None if provision_with is None else ','.join(provision_with)
+        provision_arg = None if provision is None else '--provision' if provision else '--no-provision'
+        self._run_vagrant_command('reload', vm_name, provision_arg,
+                                  prov_with_arg, providers_arg,
+                                  **kwargs)
+
     def suspend(self, vm_name=None, **kwargs):
         '''
         Suspend/save the machine.
@@ -444,11 +512,33 @@ class Vagrant(object):
         '''
         self._run_vagrant_command('box', 'remove', name, provider, **kwargs)
 
-    def provision(self, vm_name=None, **kwargs):
+    def plugin_list(self):
         '''
-        Runs the provisioners defined in the Vagrantfile.
+        Return a list of dicts containing the following information about
+        installed plugins:
+        - 'name': The plugin name, as a string.
+        - 'version': The plugin version, as a string.
+        - 'system': A boolean, presumably indicating whether this plugin is a
+          "core" part of vagrant, though the feature is undocumented.
         '''
-        self._run_vagrant_command('provision', vm_name, **kwargs)
+        output = self._run_vagrant_command('plugin', 'list')
+        return [self._parse_plugin_list_line(l) for l in output.splitlines()]
+
+    def _parse_plugin_list_line(self, line):
+        # As of Vagrant 1.5, the format of the `vagrant plugin list` command can
+        # be inferred here:
+        # https://github.com/mitchellh/vagrant/blob/master/plugins/commands/plugin/action/list_plugins.rb#L35
+        # Example plugin listing lines:
+        # sahara (0.0.16)
+        # vagrant-login (1.0.1, system)
+        regex = re.compile(r'^(?P<name>.+?)\s+\((?P<version>.+?)(?P<system>, system)?\)$')
+        m = regex.search(line)
+        if m is None:
+            raise Exception('Error parsing plugin listing line.', line)
+        else:
+            listing = m.groupdict()
+            listing['system'] = bool(listing['system'])
+            return listing
 
     def _parse_provider_line(self, line):
         '''
